@@ -7,6 +7,7 @@
 #include <kauth/helpersupport.h>
 #include <QProcess>
 #include <QRegularExpression>
+#include <KLocalizedString>
 
 static bool isValidUserName(const QString &name)
 {
@@ -19,19 +20,28 @@ ActionReply AuthHelper::isuserknown(const QVariantMap &args)
 {
     const auto username = args.value(QStringLiteral("username")).toString();
     if (!isValidUserName(username)) {
-        return ActionReply::HelperErrorReply();
+        auto reply = ActionReply::HelperErrorReply();
+        reply.setErrorDescription(xi18nc("@info", "User name <resource>%1</resource> is not valid as the name of a Samba user; cannot check for its existence.", username));
+        return reply;
     }
 
     QProcess p;
-    p.setProgram(QStringLiteral("pdbedit"));
-    p.setArguments({ QStringLiteral("--debuglevel=0"), QStringLiteral("--user"), username });
+    const auto program = QStringLiteral("pdbedit");
+    const auto arguments = QStringList({QStringLiteral("--debuglevel=0"), QStringLiteral("--user"), username });
+    p.setProgram(program);
+    p.setArguments(arguments);
     p.start();
     // Should be fairly quick: short timeout.
     const int pdbeditTimeout = 4000; // milliseconds
     p.waitForFinished(pdbeditTimeout);
 
     if (p.exitStatus() != QProcess::NormalExit) {
-        return ActionReply::HelperErrorReply();
+        // QByteArray can't do direct conversion to QString
+        const QString errorText = QString::fromUtf8(p.readAllStandardOutput());
+        auto reply = ActionReply::HelperErrorReply();
+        reply.setErrorDescription(xi18nc("@info '%1 %2' together make up a terminal command; %3 is the command's output",
+                                         "Command <command>%1 %2</command> failed:<nl/><nl/>%3", program, arguments.join(QLatin1Char(' ')), errorText));
+        return reply;
     }
 
     ActionReply reply;
@@ -43,8 +53,15 @@ ActionReply AuthHelper::createuser(const QVariantMap &args)
 {
     const auto username = args.value(QStringLiteral("username")).toString();
     const auto password = args.value(QStringLiteral("password")).toString();
-    if (!isValidUserName(username) || password.isEmpty()) {
-        return ActionReply::HelperErrorReply();
+    if (!isValidUserName(username)) {
+        auto reply = ActionReply::HelperErrorReply();
+        reply.setErrorDescription(xi18nc("@info", "<resource>%1</resource> is not a valid Samba user name; cannot create it.", username));
+        return reply;
+    }
+    if (password.isEmpty()) {
+        auto reply = ActionReply::HelperErrorReply();
+        reply.setErrorDescription(i18nc("@info", "For security reasons, creating Samba users with empty passwords is not allowed."));
+        return reply;
     }
 
     QProcess p;
@@ -81,8 +98,15 @@ ActionReply AuthHelper::addtogroup(const QVariantMap &args)
 {
     const auto user = args.value(QStringLiteral("user")).toString();
     const auto group = args.value(QStringLiteral("group")).toString();
-    if (!isValidUserName(user) || !isValidUserName(group)) {
-        return ActionReply::HelperErrorReply();
+    if (!isValidUserName(user)) {
+        auto reply = ActionReply::HelperErrorReply();
+        reply.setErrorDescription(xi18nc("@info", "<resource>%1</resource> is not a valid user name; cannot add it to the group <resource>%2</resource>.", user, group));
+        return reply;
+    }
+    if (!isValidUserName(group)) {
+        auto reply = ActionReply::HelperErrorReply();
+        reply.setErrorDescription(xi18nc("@info", "<resource>%1</resource> is not a valid group name; cannot make user <resource>%2</resource> a member of it.", group, user));
+        return reply;
     }
     // Harden against some input abuse.
     // TODO: add ability to resolve remote UID via KAuth and verify the request (or even reduce the arguments down to
@@ -90,7 +114,11 @@ ActionReply AuthHelper::addtogroup(const QVariantMap &args)
     // Keep this condition in sync with the one in groupmanager.cpp
     if (!group.contains(QLatin1String("samba")) || group.contains(QLatin1String("admin")) ||
         group.contains(QLatin1String("root"))) {
-        return ActionReply::HelperErrorReply();
+        auto reply = ActionReply::HelperErrorReply();
+        reply.setErrorDescription(xi18nc("@info", "For security reasons, cannot make user <resource>%1</resource> a member of group <resource>%2</resource>. \
+                                                   The group name is insecure; valid group names include the text <resource>samba</resource> and do not \
+                                                   include the text <resource>admin</resource> or <resource>root</resource>."));
+        return reply;
     }
 
     QProcess p;
